/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

void etherxs_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);
    int port=ts->port;

    /* this thread is invalid now */
    eh->t = NULL;

    /* someone already started a daemon, we don't need to exist */
    if(etherx__daemon != NULL) return;

    pth_sleep(eh->attempts); /* increasing delay */

    /* only try for so long */
    if(eh->attempts++ > ETHERX_MERCY)
    {
        log_error("etherxs","Failed to establish communication with etherxd");
        raise(SIGTERM);
    }

    /* try to start a daemon if we're in local mode */
    if(etherx__remote == NULL && etherxd_start())
        return;

    log_warn("etherxs","Connection to etherxd failed %d times, attempting to reconnect...",eh->attempts);
    etherxs_connect(eh,port); /* start the process all over */
}

void etherxs_receive(thread t, xmlnode x, int flag)
{
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);
    epacket ep;

    switch(flag)
    {
    case XMLSTREAM_ROOT:
        log_debug(ZONE,"got root tag from etherx!");
        eh->t = t; /* we're successfully connected, associate us w/ the ehandle */
        eh->attempts = 0;
        if(ts->port!=5269)
        { /* we are using the new method */
            char *in_buf=spools(x->p,xmlnode_get_attrib(x,"id"),etherx__secret,x->p);
            char *out_buf;
            xmlnode hs;

            log_debug(ZONE,"shahashing: %s",in_buf);
            out_buf=shahash(in_buf);
            log_debug(ZONE,"output: %s:%s",in_buf,out_buf);

            hs=xmlnode_new_tag_pool(x->p,"handshake");
            xmlnode_insert_cdata(hs,out_buf,-1);
            in_buf=xmlnode2str(hs);
            log_debug(ZONE,"writing handshake: %s",in_buf);
            tstream_write(t,in_buf,-1);
        }
        else
            eh->state=EHANDLER_AUTHED;            
        /* ignore the root tag content, transports don't use it */
        break;
    case XMLSTREAM_BOUNCE:
        ep = epacket_new(x, NULL, NULL, eh->namespace);
        log_debug(ZONE,"bouncing data intended for etherxd");
        epacket_error(ep,"delivery failure");
        (*eh->h)(ep);
        break;
    case XMLSTREAM_NODE:
        log_debug(ZONE,"got node from etherx!");
        if(eh->state==EHANDLER_NOTAUTHED)
        {
            if(j_strcmp(xmlnode_get_name(x),"handshake")==0)
            {
                eh->state=EHANDLER_AUTHED;
            }
            xmlnode_free(x);
        }
        else
        {
            if(j_strcmp(xmlnode_get_name(x),"stream:error") == 0)
            {
                log_error("etherxs","connection to etherxd was rejected: %s",xmlnode_get_data(x));
                raise(SIGTERM);
            }
            ep = epacket_new(x, eh->host, NULL, eh->namespace);
            (*eh->h)(ep);
        }
        break;
    case XMLSTREAM_INIT:
        pool_cleanup(t->p, etherxs_cleanup, (void *)t);
        break;
    }
}

void etherxs_connect(ehandler eh,int port)
{
    thread t;
    tstream ts;
    xmlnode x;
    spool s;

    log_debug(ZONE,"attempting socket connect to %s for transport %s",etherx__remote,eh->host);

    /* default to localhost unless otherwise specified */
    if(etherx__remote != NULL)
        t = xmlstream_connect(port, etherx__remote, (xmlstream_onNode)etherxs_receive);
    else
        t = xmlstream_connect(port, "127.0.0.1", (xmlstream_onNode)etherxs_receive);

    ts = (tstream)(t->data);
    ts->data_aux1 = (void *)eh; /* so that _etherx_receive can access the handler info */
    x = xmlstream_header(eh->namespace, etherx__remote, eh->host);
    xmlnode_put_attrib(x, "xmlns:etherx", ETHERX_NAMESPACE);
    s = spool_new(xmlnode_pool(x));
    spooler(s,xmlnode_get_attrib(x, "id"),etherx__secret,s);
    xmlnode_put_attrib(x, "etherx:secret", shahash(spool_print(s)));
    xmlstream_write_header(t, x);
}

